#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Bridge between OSC and MQTT."""

from __future__ import print_function, unicode_literals

import argparse
import ipaddress
import json
import logging
import sys
import time

import liblo

import paho.mqtt.client as mqtt


log = logging.getLogger('osc2mqtt')


def handle_osc(oscaddr, values, types, clientaddr, userdata):
    log.debug("OSC recv: %s %r", oscaddr, values)
    if len(values) > 1:
        msg = json.dumps(values)
    else:
        msg = values[0]
    topic = userdata['prefix'] + oscaddr
    log.debug("MQTT publish: %s %s", topic, msg)
    userdata['mqtt'].publish(topic, msg)


def on_connect(client, userdata, flags, rc):
    log.debug("MQTT connect: %s", mqtt.connack_string(rc))
    client.subscribe('#')


def on_disconnect(client, userdata, rc):
    log.debug("MQTT disconnect: %s", mqtt.error_string(rc))


def on_message(client, userdata, msg):
    log.debug("MQTT recv: %s %r", msg.topic, msg.payload)
    if userdata.get('osc_receiver'):
        try:
            values = json.loads(msg.payload.decode('utf-8'))
        except (TypeError, ValueError):
            values = [msg.payload]

        if isinstance(values, (int, float)):
            values = [values]

        addr = '/' + msg.topic.lstrip(userdata.get('prefix', '')).lstrip('/')
        log.debug("OSC send: %s %r", addr, values)
        userdata['osc'].send(userdata['osc_receiver'], addr, *values)


def parse_hostport(addr, port=9000):
    if ('::' in addr and ']:' in addr) or ('::' not in addr and ':' in addr):
        host, port = addr.rsplit(':', 1)
    else:
        host = addr

    if host.startswith('[') and host.endswith(']'):
        host = host[1:-1]

    return (host, int(port))


def main(args=None):
    ap = argparse.ArgumentParser(description=__doc__.splitlines()[0])
    ap.add_argument(
        '-p', '--osc-port',
        type=int,
        metavar='PORT',
        default=9001,
        help="Local OSC server (UDP) port (default: %(default)s)")
    ap.add_argument(
        '-m', '--mqtt-broker',
        metavar='ADDR[:PORT]',
        default='localhost:1883',
        help="MQTT broker addr[:port] (default: %(default)s)")
    ap.add_argument(
        '-o', '--osc-receiver',
        metavar='ADDR[:PORT]',
        help='Also bridge MQTT to OSC receiver addr[:port] via UDP '
             '(default: one-way)')
    ap.add_argument(
        '-t', '--topic-prefix',
        default='',
        metavar='PREFIX',
        help='MQTT topic prefix to prepend to/strip from OSC addresses')
    ap.add_argument(
        '-v', '--verbose',
        action="store_true",
        help='Enable verbose logging')

    args = ap.parse_args(args if args is not None else sys.argv[1:])

    logging.basicConfig(level=logging.DEBUG if args.verbose else logging.INFO)

    userdata = {'prefix': args.topic_prefix}

    if args.osc_receiver:
        osc_host, osc_port = parse_hostport(args.osc_receiver, 9000)
        userdata['osc_receiver'] = liblo.Address(osc_host, osc_port, liblo.UDP)

    mqttclient = mqtt.Client("osc2mqtt", userdata=userdata)
    userdata['mqtt'] = mqttclient
    mqttclient.on_connect = on_connect
    mqttclient.on_disconnect = on_disconnect
    mqttclient.on_message = on_message
    oscserver = liblo.ServerThread(args.osc_port)
    userdata['osc'] = oscserver
    oscserver.add_method(None, None, handle_osc, userdata)

    mqtt_host, mqtt_port = parse_hostport(args.mqtt_broker, 1883)
    log.info("Connecting to MQTT broker %s:%s ...", mqtt_host, mqtt_port)
    mqttclient.connect(mqtt_host, mqtt_port)

    log.debug("Starting MQTT thread...")
    mqttclient.loop_start()

    log.info("Starting OSC server listening on port %s ...", args.osc_port)
    oscserver.start()

    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        log.info("Interrupted.")
    finally:
        log.info("Stopping OSC server ...")
        oscserver.stop()
        log.debug("Stopping MQTT thread ...")
        mqttclient.loop_stop()
        log.info("Disconnecting from MQTT broker ...")
        mqttclient.disconnect()
        log.info("Done.")


if __name__ == '__main__':
    sys.exit(main(sys.argv[1:]) or 0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# setup.py - Setup file for the osc2mqtt project.
#
"""An OSC to MQTT bridge based on pyliblo and paho-mqtt."""

from io import open
from setuptools import setup

# Add custom distribution meta-data, avoids warning when running setup
from distutils.dist import DistributionMetadata
DistributionMetadata.repository = None

classifiers = """\
Development Status :: 4 - Beta
Environment :: Console
Intended Audience :: Developers
Intended Audience :: End Users/Desktop
Intended Audience :: Manufacturing
Intended Audience :: Other Audience
License :: OSI Approved :: MIT License
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: MacOS :: MacOS X
Programming Language :: Python
Programming Language :: Python :: 2.7
Programming Language :: Python :: 3.3
Programming Language :: Python :: 3.4
Topic :: Communications
Topic :: Internet
Topic :: Home Automation
Topic :: Multimedia :: Sound/Audio
"""
name = 'osc2mqtt'
url = 'https://github.com/SpotlightKid/%s' % name.lower()
readme = open('README.rst', encoding='utf-8').read()

setup(
    name = name,
    version = '0.1b1',
    description = __doc__.splitlines()[0],
    long_description = "\n".join(readme.splitlines()[2:]),
    keywords = 'osc mqtt iot',
    classifiers = [c.strip() for c in classifiers.splitlines()
        if c.strip() and not c.startswith('#')],
    author = 'Christopher Arndt',
    author_email = 'chris@chrisarndt.de',
    url = url,
    repository = 'https://github.com/SpotlightKid/%s' % name.lower(),
    download_url = url + '/releases',
    license = 'MIT License',
    platforms = 'POSIX, Windows, MacOS X',
    py_modules = ['osc2mqtt'],
    install_requires = [
        'paho-mqtt',
        'pyliblo',
    ],
    entry_points = {
        'console_scripts': [
            'osc2mqtt = osc2mqtt:main'
        ]
    },
    zip_safe = True
)
